@@ -247,7 +247,6 @@ gb_editor_finalize_impl (GObject *o)
 	gb_editor_set_location_source (e, NULL);
 	
 	if (p->btree_model) g_object_unref (p->btree_model);
-	if (p->main_widget) g_object_unref (p->main_widget);
 	if (p->clipboard) g_object_unref (p->clipboard);
 	if (p->set) g_object_unref (p->set);
 	if (p->tree_view) g_object_unref (p->tree_view);
@@ -335,6 +334,8 @@ gb_editor_init_widgets_impl (GbEditor *e)
 	mw = bonobo_window_new ("Galeon Bookmarks Editor", "Bookmarks Editor");
 	gb_editor_set_main_widget (e, mw);
 
+	g_object_weak_ref (G_OBJECT (mw), (GWeakNotify) g_object_unref, e);
+
 	/* makes sure the tree view has been created */
 	gb_editor_get_tree_view (e);
 	
