@@ -161,7 +161,7 @@ gb_import_druid_init_widgets (GbImportDruid *d)
 	gtk_window_set_title (GTK_WINDOW (p->window), 
 			      gb_import_druid_get_string (d, "window title"));
 
-	g_signal_connect_swapped (p->window, "delete_event", G_CALLBACK (g_object_unref), d);
+	g_object_weak_ref (G_OBJECT (p->window), (GWeakNotify) g_object_unref, d);
 
 	p->druid = GNOME_DRUID (gnome_druid_new ());
 	gtk_widget_show (GTK_WIDGET (p->druid));
