@@ -18,19 +18,23 @@
 
 #include "galeon-embed-popup.h"
 #include "galeon-embed-event.h"
+#include "galeon-embed-shell.h"
 #include "galeon-embed-utils.h"
 #include "prefs-strings.h"
 #include "eel-gconf-extensions.h"
 #include "galeon-config.h"
 #include "gul-bonobo-extensions.h"
 #include "gul-download.h"
+#include "gul-string.h"
 #include "galeon-embed-helper-list.h"
 #include "galeon-marshal.h"
 
 #include <string.h>
+#include <bonobo/bonobo-i18n.h>
 #include <bonobo/bonobo-ui-component.h>
 #include <gtk/gtkclipboard.h>
 #include <libgnome/gnome-exec.h>
+#include <libgnomevfs/gnome-vfs-uri.h>
 
 typedef enum
 {
@@ -136,6 +140,12 @@ static gint GaleonEmbedPopupSignals[GALEON_EMBED_POPUP_LAST_SIGNAL];
 #define GALEON_POPUP_IMAGE_OPENWITH_PATH "/popups/GaleonEmbedElementPopup/ImageItems/EPOpenImageWith"
 #define GALEON_POPUP_OPEN_IMAGE_PATH "/commands/EPOpenImage"
 
+/*This command is from src/popup-commands.c but manipulating item label
+ *text must be done here
+ */
+#define GALEON_POPUP_BLOCK_IMAGE_SITE_PATH "/commands/EPBlockImageSite"
+#define GALEON_POPUP_ALLOW_IMAGE_SITE_PATH "/commands/EPAllowImageSite"
+
 BonoboUIVerb embed_popup_verbs [] = {
 	BONOBO_UI_VERB ("EPCopyLinkLocation", (BonoboUIVerbFn)embed_popup_copy_link_location_cmd),
 	BONOBO_UI_VERB ("EPDownloadLink", (BonoboUIVerbFn)embed_popup_download_link_cmd),
@@ -231,6 +241,11 @@ galeon_embed_popup_finalize (GObject *object)
 		g_object_unref (G_OBJECT (gep->priv->event));
 	}	
 	
+	if (gep->priv->helper_list)
+	{
+		g_object_unref (G_OBJECT (gep->priv->helper_list));
+	}
+	
         g_free (gep->priv);
 	
         G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -291,7 +306,81 @@ setup_element_menu (GaleonEmbedPopup *p)
 	
 	if (is_image)
 	{
-		setup_image_openwith (p);
+		GaleonEmbedEvent *info;
+		const GValue *value;
+		char *open_image_str, *block_host_str,
+		     *esc_file = NULL, *file = NULL;
+		const char *host = NULL;
+		GnomeVFSURI *uri = NULL;
+		const char *show_path, *hide_path;
+		PermissionActionType perm_action;
+
+		info = galeon_embed_popup_get_event(p);
+
+		galeon_embed_event_get_property(info, "image", &value);
+
+		uri = gnome_vfs_uri_new(g_value_get_string(value));
+
+		if (uri)
+		{
+			host = gnome_vfs_uri_get_host_name(uri);
+			file = gnome_vfs_uri_extract_short_name(uri);
+		}
+
+		if (!host)
+			host = _("This Site");
+
+		if (file)
+		{
+			esc_file = gul_string_escape_uline_accel(file);
+			g_free(file);
+		}
+
+		galeon_embed_shell_test_permission(embed_shell,
+						   g_value_get_string(value),
+						   IMAGES_PERMISSION,
+						   &perm_action);
+
+		if (perm_action == DENY_ACTION)
+		{
+			block_host_str = g_strdup_printf(_("Allow Images From %s"),
+							 host);
+			show_path = GALEON_POPUP_ALLOW_IMAGE_SITE_PATH;
+			hide_path = GALEON_POPUP_BLOCK_IMAGE_SITE_PATH;
+		}
+		else
+		{
+			block_host_str = g_strdup_printf(_("Block Images From %s"),
+							 host);
+			hide_path = GALEON_POPUP_ALLOW_IMAGE_SITE_PATH;
+			show_path = GALEON_POPUP_BLOCK_IMAGE_SITE_PATH;
+		}
+
+		gul_bonobo_set_label(p->priv->ui_component,
+				     show_path,
+				     block_host_str);
+		gul_bonobo_set_hidden(p->priv->ui_component,
+				      show_path, FALSE);
+		gul_bonobo_set_hidden(p->priv->ui_component,
+				      hide_path, TRUE);
+
+		open_image_str = g_strdup_printf(_("Open Image (%s)"),
+						 esc_file);
+
+		gul_bonobo_set_label(p->priv->ui_component,
+				     GALEON_POPUP_OPEN_IMAGE_PATH,
+				     open_image_str);
+
+		if (esc_file)
+			g_free(esc_file);
+
+		if (uri)
+			gnome_vfs_uri_unref(uri);
+
+		g_free(block_host_str);
+		g_free(open_image_str);
+
+		setup_image_openwith(p);
 	}
 }
 
@@ -523,6 +612,8 @@ save_property_url (GaleonEmbedPopup *popup,
 				 ask_dest,
                                  FALSE,
                                  persist);
+
+	g_object_unref (persist);
 }
 
 const char *
@@ -690,6 +781,8 @@ save_url (GaleonEmbedPopup *popup,
 				 ask_dest,
                                  FALSE,
                                  persist);
+
+	g_object_unref (persist);
 }
 
 static void 
