@@ -267,8 +267,16 @@ GContentHandler::~GContentHandler()
 
 /* void show (in nsIHelperAppLauncher aLauncher, in nsISupports aContext); */
 NS_IMETHODIMP GContentHandler::Show(nsIHelperAppLauncher *aLauncher,
-				    nsISupports *aContext)
+				    nsISupports *aContext
+#if MOZILLA_SNAPSHOT > 9
+, PRBool aForced)
+#else
+)
+#endif
 {
+	/* aForced reflects if the content being sent is normally viewable
+	 * in mozilla or not. That fact doesn't affect us, so ignore it
+	 */
 	nsresult rv;
 
 	mLauncher = aLauncher;
@@ -404,6 +412,7 @@ NS_IMETHODIMP GContentHandler::
 	}
 }
 
+#if MOZILLA_SNAPSHOT <= 9
 /* void showProgressDialog (in nsIHelperAppLauncher aLauncher, in nsISupports aContext); */
 NS_METHOD GContentHandler::ShowProgressDialog(nsIHelperAppLauncher *aLauncher,
 					      nsISupports *aContext)
@@ -411,6 +420,7 @@ NS_METHOD GContentHandler::ShowProgressDialog(nsIHelperAppLauncher *aLauncher,
 	g_print ("GContentHandler::ShowProgressDialog is depreciated!\n");
 	return NS_ERROR_NOT_IMPLEMENTED;
 }
+#endif
 
 ////////////////////////////////////////////////////////////////////////////////
 // begin local public methods impl
@@ -480,12 +490,6 @@ NS_METHOD GContentHandler::LaunchHelperApp (void)
 	return NS_OK;
 }
 
-NS_METHOD GContentHandler::ShowHelperProgressDialog (void)
-{
-	mHelperProgress = PR_TRUE;
-	return ShowProgressDialog (mLauncher,mContext);
-}
-
 NS_METHOD GContentHandler::GetLauncher (nsIHelperAppLauncher * *_retval)
 {
 	NS_IF_ADDREF (*_retval = mLauncher);
