@@ -15,6 +15,9 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include "EventContext.h"
 #include "nsIDOMEventTarget.h"
@@ -39,10 +42,13 @@
 #include <gdk/gdkkeysyms.h>
 #include "nsIPrivateDOMEvent.h"
 #include "nsIDOMNSUIEvent.h"
-#include <nsIDOMNSHTMLDocument.h>
+#include <nsISelection.h>
 
 #define KEY_CODE 256
 
+static nsresult ResolveBaseURL (nsIDocument *doc, const nsAString &relurl, nsACString &url);
+static nsresult ResolveDocumentURL (nsIDocument *doc, const nsAString &relurl, nsACString &url);
+
 EventContext::EventContext ()
 {
 }
@@ -51,9 +57,8 @@ EventContext::~EventContext ()
 {
 }
 
-nsresult EventContext::Init (nsIDOMEvent *event, GaleonWrapper *wrapper)
+nsresult EventContext::Init (GaleonWrapper *wrapper)
 {
-	mEvent = event;
 	mWrapper = wrapper;
 	mDOMDocument = nsnull;
 
@@ -98,14 +103,20 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 	// do something useful with it.
 	
 	// Get the selected text in the document, if any
-	nsCOMPtr<nsIDOMNSHTMLDocument> htmlDoc = do_QueryInterface(domDoc, &rv);
+	nsCOMPtr<nsIDOMWindow> domWindow;
+	rv = mWrapper->GetDOMWindow(getter_AddRefs(domWindow));
 	if (NS_SUCCEEDED(rv))
 	{
-		nsAutoString selection;
-		(void)htmlDoc->GetSelection (selection);
-		if (!selection.IsEmpty())
+		nsCOMPtr<nsISelection> selection;
+		domWindow->GetSelection(getter_AddRefs(selection));
+		if (NS_SUCCEEDED(rv))
 		{
-			SetStringProperty ("selection", selection);
+			nsXPIDLString selText;
+			rv = selection->ToString(getter_Copies(selText));
+			if (NS_SUCCEEDED(rv) && !selText.IsEmpty())
+			{
+				SetStringProperty ("selection", selText);
+			}
 		}
 	}
 
@@ -144,12 +155,7 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 			if (NS_SUCCEEDED(rv) && !img.IsEmpty())
 			{
 				nsCAutoString imglongdesc;
-				const nsACString &src = NS_ConvertUCS2toUTF8(img);
-
-				nsCOMPtr<nsIURI> uri;
-				doc->GetDocumentURL(getter_AddRefs(uri));
-
-				rv = uri->Resolve (src, imglongdesc);
+				rv = ResolveDocumentURL (doc, img, imglongdesc);
 
 				SetStringProperty ("image_long_desc",
 						   NS_ConvertUTF8toUCS2(imglongdesc));
@@ -192,13 +198,10 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 				if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
 
 				nsCAutoString cImg;
-				const nsACString &src = NS_ConvertUCS2toUTF8(img);
+				rv = ResolveDocumentURL (doc, img, cImg);
+				if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
 
-				nsCOMPtr<nsIURI> uri;
-				doc->GetDocumentURL(getter_AddRefs(uri));
-                                rv = uri->Resolve (src, cImg);
-				SetStringProperty ("image",
-						   NS_ConvertUTF8toUCS2(cImg));
+				SetStringProperty ("image", cImg.get());
 
 				if (NS_FAILED (rv)) return NS_ERROR_FAILURE;
 			}
@@ -247,15 +250,10 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 				if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
 				
 				nsCAutoString cImg;
-				const nsACString &src = NS_ConvertUCS2toUTF8(img);
-
-				nsCOMPtr<nsIURI> uri;
-				doc->GetDocumentURL(getter_AddRefs(uri));
-				rv = uri->Resolve (src, cImg);
-				SetStringProperty ("image",
-						   NS_ConvertUTF8toUCS2(cImg));
-
+				rv = ResolveDocumentURL (doc, img, cImg);
 				if (NS_FAILED (rv)) return NS_ERROR_FAILURE;
+
+				SetStringProperty ("image", cImg.get());
 			}
 			else
 			{
@@ -420,16 +418,10 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 			if (!value.IsEmpty())
 			{
 				nsCAutoString bgimg;
-				const nsACString &tmp =
-					NS_ConvertUCS2toUTF8(value);
+				rv = ResolveDocumentURL (doc, value, bgimg);
+				if (NS_FAILED(rv)) return NS_ERROR_FAILURE;
 
-				nsCOMPtr<nsIURI> uri;
-				doc->GetDocumentURL(getter_AddRefs(uri));
-				rv = uri->Resolve (tmp, bgimg);
-				if (NS_FAILED (rv))
-					return NS_ERROR_FAILURE;
-				SetStringProperty ("background_image",
-						   NS_ConvertUTF8toUCS2(bgimg));
+				SetStringProperty ("background_image", bgimg.get());
 			}
 			else
 			{
@@ -443,17 +435,12 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 					if (!value.IsEmpty())
 					{
 						nsCAutoString bgimg;
-						const nsACString &tmp =
-							NS_ConvertUCS2toUTF8(value);
-
-						nsIURI *uri;
-						doc->GetBaseURL(uri);
-						rv = uri->Resolve 
-							(tmp, bgimg);
-						SetStringProperty ("background_image",
-						   NS_ConvertUTF8toUCS2(bgimg));
-						if (NS_FAILED (rv))
+						rv = ResolveBaseURL (doc, value, bgimg);
+						if (NS_FAILED(rv))
 							return NS_ERROR_FAILURE;
+
+						SetStringProperty ("background_image",
+								   bgimg.get());
 						has_background = PR_TRUE;
 					}
 				}
@@ -466,17 +453,12 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 				if (NS_SUCCEEDED (rv))
 				{
 					nsCAutoString bgimg;
-					const nsACString &tmp =
-						NS_ConvertUCS2toUTF8(cssurl);
-
-					nsIURI *uri;
-					doc->GetBaseURL(uri);
-					rv = uri->Resolve 
-						(tmp, bgimg);
-					SetStringProperty ("background_image",
-						   NS_ConvertUTF8toUCS2(bgimg));
+					rv = ResolveBaseURL (doc, cssurl, bgimg);
 					if (NS_FAILED (rv))
 						return NS_ERROR_FAILURE;
+
+					SetStringProperty ("background_image",
+							   bgimg.get());
 				}
 			}
 		}
@@ -535,10 +517,11 @@ nsresult EventContext::GetCSSBackground (nsIDOMNode *node, nsAutoString& url)
 	return NS_OK;
 }
 
-nsresult EventContext::GetMouseEventInfo (GaleonEmbedEvent *info)
+nsresult EventContext::GetMouseEventInfo (nsIDOMMouseEvent *aMouseEvent, GaleonEmbedEvent *info)
 {
 	nsresult result;
-	nsIDOMMouseEvent *aMouseEvent = (nsIDOMMouseEvent*)mEvent;
+
+	info->keycode = 0;
 
 	// casting 32-bit guint* to PRUint16* below will break on big-endian
 	PRUint16 btn;
@@ -546,8 +529,8 @@ nsresult EventContext::GetMouseEventInfo (GaleonEmbedEvent *info)
 	info->mouse_button = (guint)btn;
 
 	// OTOH, casting only between (un)signedness is safe
-	aMouseEvent->GetScreenX ((PRInt32*)&info->mouse_x);
-	aMouseEvent->GetScreenY ((PRInt32*)&info->mouse_y);
+	aMouseEvent->GetScreenX ((PRInt32*)&info->x);
+	aMouseEvent->GetScreenY ((PRInt32*)&info->y);
 
 	/* be sure we are not clicking on the scroolbars */
 
@@ -604,6 +587,66 @@ nsresult EventContext::GetMouseEventInfo (GaleonEmbedEvent *info)
 	return NS_OK;
 }
 
+nsresult EventContext::GetKeyEventInfo (nsIDOMKeyEvent *aKeyEvent, GaleonEmbedEvent *info)
+{
+	nsresult rv;
+
+	info->mouse_button = 0;
+
+	PRUint32 keyCode;
+	rv = aKeyEvent->GetKeyCode(&keyCode);
+	if (NS_FAILED(rv)) return rv;
+	info->keycode = keyCode;
+
+	nsCOMPtr<nsIDOMEventTarget> target;
+	rv = aKeyEvent->GetTarget(getter_AddRefs(target));
+	if (NS_FAILED(rv) || !target) return NS_ERROR_FAILURE;
+
+	/* Calculate the node coordinates relative to the widget origin */
+	nsCOMPtr<nsIDOMNSHTMLElement> elem = do_QueryInterface(target, &rv);
+	if (NS_FAILED(rv)) return rv;
+
+	PRInt32 x = 0, y = 0;
+	while (elem)
+	{
+		PRInt32 val;
+		elem->GetOffsetTop(&val);	y += val;
+		elem->GetScrollTop(&val);	y -= val;
+		elem->GetOffsetLeft(&val);	x += val;
+		elem->GetScrollLeft(&val);	x -= val;
+
+		nsCOMPtr<nsIDOMElement> parent;
+		elem->GetOffsetParent(getter_AddRefs(parent));
+		elem = do_QueryInterface(parent, &rv);
+	}
+	info->x = x;
+	info->y = y;
+
+	/* Context */
+	rv = GetEventContext (target, info);
+	if (NS_FAILED(rv)) return rv;
+
+	/* Get the modifier */
+
+	PRBool mod_key;
+
+	info->modifier = 0;
+
+	aKeyEvent->GetAltKey(&mod_key);
+	if (mod_key) info->modifier |= GDK_MOD1_MASK;
+
+	aKeyEvent->GetShiftKey(&mod_key);
+	if (mod_key) info->modifier |= GDK_SHIFT_MASK;
+
+	aKeyEvent->GetMetaKey(&mod_key);
+	if (mod_key) info->modifier |= GDK_Meta_L;
+	
+	aKeyEvent->GetCtrlKey(&mod_key);
+	if (mod_key) info->modifier |= GDK_CONTROL_MASK;
+
+	return NS_OK;
+}
+
 nsresult EventContext::IsPageFramed (nsIDOMNode *node, PRBool *Framed)
 {
 	nsresult result;
@@ -641,9 +684,7 @@ nsresult EventContext::SetIntProperty (const char *name, int value)
 	
 	g_value_set_int (val, value);
 
-	galeon_embed_event_set_property (mEmbedEvent, 
-			     		 g_strdup (name),
-			     		 val);
+	galeon_embed_event_set_property (mEmbedEvent, name, val);
 
 	return NS_OK;
 }
@@ -656,28 +697,36 @@ nsresult EventContext::SetStringProperty (const char *name, const char *value)
 	
 	g_value_set_string (val, value);
 			 
-	galeon_embed_event_set_property (mEmbedEvent, 
-			     		 g_strdup (name),
-			     		 val);
+	galeon_embed_event_set_property (mEmbedEvent, name, val);
 
 	return NS_OK;
 }
 
 nsresult EventContext::SetStringProperty (const char *name, const nsAString &value)
 {
-	GValue *val = g_new0 (GValue, 1);;
-	char *tmp;
-	
-	tmp = ToNewCString (NS_ConvertUCS2toUTF8(value));
-	
-	g_value_init (val, G_TYPE_STRING);
-
-	g_value_set_string (val, tmp);
-	
-	galeon_embed_event_set_property (mEmbedEvent, 
-			     		 g_strdup (name),
-			     		 val);
-	nsMemory::Free (tmp);
-
-	return NS_OK;
+	return SetStringProperty (name, NS_ConvertUCS2toUTF8(value).get());
+}
+
+nsresult ResolveBaseURL (nsIDocument *doc, const nsAString &relurl, nsACString &url)
+{
+	nsresult rv;
+	nsCOMPtr<nsIURI> base;
+#if MOZILLA_SNAPSHOT > 9
+	rv = doc->GetBaseURL (getter_AddRefs(base));
+#else
+	rv = doc->GetBaseURL (*getter_AddRefs(base));
+#endif
+	if (NS_FAILED(rv)) return rv;
+
+	return base->Resolve (NS_ConvertUCS2toUTF8(relurl), url);
+}
+
+nsresult ResolveDocumentURL (nsIDocument *doc, const nsAString &relurl, nsACString &url)
+{
+	nsresult rv;
+	nsCOMPtr<nsIURI> uri;
+	rv = doc->GetDocumentURL(getter_AddRefs(uri));
+	if (NS_FAILED(rv)) return rv;
+
+	return uri->Resolve (NS_ConvertUCS2toUTF8(relurl), url);
 }
