@@ -48,8 +48,6 @@
 #include "nsIScriptContext.h"
 
 #include "nsIDOMWindowInternal.h"
-#include "nsICharsetConverterManager.h"
-#include "nsICharsetConverterManager2.h"
 #include "nsIInterfaceRequestor.h"
 #include "nsIFocusController.h"
 #include "nsIWebBrowserPersist.h"
@@ -768,7 +766,11 @@ nsresult GaleonWrapper::ForceCharacterSet (const char *charset)
 								  &result);
 	if (NS_FAILED(result) || !mdv) return NS_ERROR_FAILURE;
 
+#if MOZILLA_SNAPSHOT > 9
+	result = mdv->SetForceCharacterSet (nsDependentCString(charset));
+#else
 	result = mdv->SetForceCharacterSet (NS_ConvertUTF8toUCS2(charset).get());
+#endif
 
 	return result;
 }
