@@ -15,6 +15,9 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "gul-glade.h"
 #include "PromptService.h"
@@ -121,6 +124,7 @@ NS_IMETHODIMP CPromptService::AlertCheck (nsIDOMWindow *parent,
 	set_check_button (check_button, checkValue);
 
 	set_title (dialog, dialogTitle);
+	gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);
 
 	gtk_dialog_run (GTK_DIALOG (dialog));
 	get_check_button (check_button, checkValue);
