@@ -54,7 +54,11 @@
 #include <nsIJVMManager.h>
 #include <nsIAtom.h>
 #include <nsICharsetConverterManager.h>
+#if MOZILLA_SNAPSHOT > 9
+#include <nsIStringEnumerator.h>
+#else
 #include <nsICharsetConverterManager2.h>
+#endif
 #include <nsIFontList.h>
 #include <nsISupportsPrimitives.h>
 #include <nsReadableUtils.h>
@@ -68,10 +72,19 @@
 #include <nsCPasswordManager.h>
 #include <nsIObserverService.h>
 
+//For setting the locale. hopefully gtkmozembed will do itself soon
+#include <chrome/nsIChromeRegistry.h>
+#include <locale/nsILocaleService.h>
+
 #define MOZILLA_PROFILE_DIR  GALEON_DIR"/mozilla"
 #define MOZILLA_PROFILE_NAME "galeon"
 #define MOZILLA_PROFILE_FILE "prefs.js"
 
+#if MOZILLA_SNAPSHOT > 9
+//Table of permission types to map to new string based api
+const char* gPermType[3] = { "cookie", "image", "popup" };
+#endif
+
 static void
 mozilla_embed_shell_class_init (MozillaEmbedShellClass *klass);
 static void
@@ -490,6 +503,41 @@ mozilla_init_cookie_observer(void)
 }
 #endif
 
+static nsresult
+getUILang(nsAString& aUILang)
+{
+	nsresult       result;
+	// get a locale service
+	nsCOMPtr<nsILocaleService> localeService = do_GetService(NS_LOCALESERVICE_CONTRACTID, &result);
+	NS_ASSERTION(NS_SUCCEEDED(result),"getUILang: get locale service failed");
+
+	nsXPIDLString uiLang;
+	result = localeService->GetLocaleComponentForUserAgent(getter_Copies(uiLang));
+	aUILang = uiLang;
+	return result;
+}
+ 
+// update global locale if possible (in case when user-*.rdf can be updated)
+// so that any apps after this can be invoked in the UILocale and contentLocale
+static nsresult mozilla_init_chrome(void)
+{
+	nsresult rv = NS_OK;
+	nsAutoString uiLang;
+	// compute lang code
+	rv = getUILang(uiLang);
+
+	nsCOMPtr<nsIXULChromeRegistry> chromeRegistry = do_GetService(NS_CHROMEREGISTRY_CONTRACTID, &rv);
+	if (chromeRegistry) {
+		chromeRegistry->SetRuntimeProvider(PR_TRUE);
+		rv = chromeRegistry->SelectLocale(NS_ConvertUCS2toUTF8(uiLang), PR_FALSE);
+
+		// Make sure skin is classic so we get native scrollbars.
+		rv = chromeRegistry->SelectSkin(NS_LITERAL_CSTRING("classic/1.0"), PR_FALSE);
+	}
+
+	return NS_OK;
+}
+
 static void
 mozilla_embed_shell_init (MozillaEmbedShell *mes)
 {
@@ -509,6 +557,9 @@ mozilla_embed_shell_init (MozillaEmbedShell *mes)
 	/* Fire up the best */
 	gtk_moz_embed_push_startup ();
 
+	/* Until gtkmozembed does this itself */
+	mozilla_init_chrome();
+
 	/* Post initialization */
 	mozilla_notifiers_init (mes);
 
@@ -701,6 +752,62 @@ impl_shell_show_js_console (GaleonEmbedShell *shell)
 	return G_OK;
 }
 
+#if MOZILLA_SNAPSHOT > 9
+static gresult
+fill_charsets_lists (MozillaEmbedShellPrivate *priv)
+{
+	nsresult rv;
+
+	nsCOMPtr<nsICharsetConverterManager> ccm =
+		do_GetService (NS_CHARSETCONVERTERMANAGER_CONTRACTID, &rv);
+	if (NS_FAILED(rv)) return G_FAILED;
+
+	nsCOMPtr<nsIUTF8StringEnumerator> e;
+	rv = ccm->GetDecoderList (getter_AddRefs(e));
+	if (NS_FAILED(rv)) return G_FAILED;
+
+        PRUint32 translated_cscount = get_translated_cscount ();
+        priv->charsets_hash = g_hash_table_new (g_str_hash, g_str_equal);
+
+	PRBool more;
+	for (rv = e->HasMore(&more); more; rv = e->HasMore(&more))
+	{
+		if (NS_FAILED(rv)) return G_FAILED;
+
+		nsCAutoString charset;
+		rv = e->GetNext(charset);
+		if (NS_FAILED(rv)) return G_FAILED;
+
+		nsAutoString title;
+		rv = ccm->GetCharsetTitle (charset.get(), title);
+		if (NS_FAILED(rv)) return G_FAILED;
+
+		char *charset_title_str = g_strdup(NS_ConvertUCS2toUTF8(title).get());
+		for (PRUint32 j = 0; j < translated_cscount; j++)
+                {
+                        if (g_ascii_strcasecmp (
+                                charset.get(), 
+                                charset_trans_array[j].charset_name) == 0)
+                        {
+                                g_free (charset_title_str);
+                                charset_title_str = (char *) 
+                                        _(charset_trans_array[j].charset_title);
+                                break;
+                        }
+                }
+
+		/* fill the hash and the sorted list */
+		g_hash_table_insert (priv->charsets_hash, charset_title_str, 
+				     g_strdup (charset.get()));
+                priv->sorted_charsets_titles = 
+			g_list_insert_sorted (priv->sorted_charsets_titles,
+                                              (gpointer)charset_title_str,
+                                              (GCompareFunc)g_ascii_strcasecmp); 
+	}
+
+	return G_OK;
+}
+#else
 static gresult
 fill_charsets_lists (MozillaEmbedShellPrivate *priv)
 {
@@ -778,6 +885,7 @@ fill_charsets_lists (MozillaEmbedShellPrivate *priv)
 
 	return G_OK;
 }
+#endif
 
 static void
 ensure_charsets_tables (MozillaEmbedShell *shell)
@@ -903,7 +1011,12 @@ impl_set_permission(GaleonEmbedShell *shell,
 	nsCOMPtr<nsIURI> uri;
 	rv = NS_NewURI(getter_AddRefs(uri), url);
 	if (NS_FAILED(rv) || !uri) return G_FAILED;
+
+	#if MOZILLA_SNAPSHOT > 9
+	rv = permissionManager->Add(uri, gPermType[type],
+	#else
 	rv = permissionManager->Add(uri, type,
+	#endif
 				    allow ? (PRUint32)nsIPermissionManager::ALLOW_ACTION
 				    	  : (PRUint32)nsIPermissionManager::DENY_ACTION);
 #else
@@ -939,13 +1052,19 @@ impl_list_permissions (GaleonEmbedShell *shell,
                 result = permissionEnumerator->GetNext(getter_AddRefs(nsPermission));
                 if (NS_FAILED(result)) return G_FAILED;
 
-#if MOZILLA_SNAPSHOT > 6
-		PRUint32 cType;
+#if MOZILLA_SNAPSHOT > 9
+		nsCAutoString cType;
+		nsPermission->GetType(cType);
+		if (cType.Equals(gPermType[type]))
 #else
+	#if MOZILLA_SNAPSHOT > 6
+		PRUint32 cType;
+	#else
                 PRInt32 cType;
-#endif
+	#endif
                 nsPermission->GetType(&cType);
                 if ((PRUint32)cType == (PRUint32)type)
+#endif
                 {
                         PermissionInfo *b = g_new0(PermissionInfo, 1);
 #if MOZILLA_SNAPSHOT > 6
@@ -997,7 +1116,11 @@ impl_remove_permissions(GaleonEmbedShell *shell,
         {
                 PermissionInfo *b = (PermissionInfo *)l->data;
                 result = permissionManager->Remove(nsDependentCString(b->domain),
+#if MOZILLA_SNAPSHOT > 9
+						   gPermType[type]);
+#else
                 				   type);
+#endif
                 if (NS_FAILED(result)) return G_FAILED;
         };
 
@@ -1020,7 +1143,11 @@ impl_test_permission (GaleonEmbedShell *shell,
 	if (NS_FAILED(rv) || !uri) return G_FAILED;
 
 	PRUint32 action;
+#if MOZILLA_SNAPSHOT > 9
+	rv = permissionManager->TestPermission(uri, gPermType[type], &action);
+#else
 	rv = permissionManager->TestPermission(uri, type, &action);
+#endif
 
 	switch (action)
 	{
@@ -1035,11 +1162,11 @@ impl_test_permission (GaleonEmbedShell *shell,
 	  	break;
 	}
 #else
-	PRBool allow;
+	PRBool blocked = PR_FALSE;
         rv = permissionManager->TestForBlocking(nsDependentCString(url),
-					        type, &allow);
+					        type, &blocked);
 
-	*ret_action = allow ? ALLOW_ACTION : DENY_ACTION;
+	*ret_action = blocked ?  DENY_ACTION : ALLOW_ACTION;
 #endif
 	if (NS_FAILED(rv)) return G_FAILED;
 
