@@ -20,8 +20,8 @@
 #include "config.h"
 #endif
 
-#define DEBUG_MSG(x) g_print x
-//#define DEBUG_MSG(x)
+//#define DEBUG_MSG(x) g_print x
+#define DEBUG_MSG(x)
 
 #define NOT_IMPLEMENTED g_warning ("not implemented: " G_STRLOC);
 
@@ -56,6 +56,7 @@ struct GaleonTabPrivate
 	GaleonEmbed *embed;
 	GaleonWindow *window;
 	gboolean is_active;
+	gboolean closing;
 	TabLoadStatus load_status;
 	char *link_message;
 	char *js_message;
@@ -111,8 +112,6 @@ galeon_tab_new_window_cb (GaleonEmbed *embed, GaleonEmbed **new_embed,
 static void
 galeon_tab_visibility_cb (GaleonEmbed *embed, gboolean visibility,
 			  GaleonTab *tab);
-static void
-galeon_tab_destroy_brsr_cb (GaleonEmbed *embed, GaleonTab *tab);
 static gint
 galeon_tab_open_uri_cb (GaleonEmbed *embed, const char *uri,
 			GaleonTab *tab);
@@ -128,6 +127,10 @@ galeon_tab_dom_mouse_down_cb (GaleonEmbed *embed,
 			      GaleonEmbedEvent *event,
 			      GaleonTab *tab);
 static void
+galeon_tab_contextmenu_cb (GaleonEmbed *embed,
+		           GaleonEmbedEvent *event,
+			   GaleonTab *tab);
+static void
 galeon_tab_security_change_cb (GaleonEmbed *embed, EmbedSecurityLevel level,
 			       GaleonTab *tab);
 static void
@@ -303,8 +306,8 @@ galeon_tab_init (GaleonTab *tab)
 	g_signal_connect (embed, "ge_visibility",
 			  GTK_SIGNAL_FUNC (galeon_tab_visibility_cb), 
 			  tab);
-	g_signal_connect (embed, "ge_destroy_brsr",
-			  GTK_SIGNAL_FUNC (galeon_tab_destroy_brsr_cb), 
+	g_signal_connect_swapped (embed, "ge_destroy_brsr",
+			  GTK_SIGNAL_FUNC (galeon_tab_destroy), 
 			  tab);
 	g_signal_connect (embed, "ge_open_uri",
 			  GTK_SIGNAL_FUNC (galeon_tab_open_uri_cb), 
@@ -318,6 +321,9 @@ galeon_tab_init (GaleonTab *tab)
 	g_signal_connect (embed, "ge_dom_mouse_down",
 			  GTK_SIGNAL_FUNC (galeon_tab_dom_mouse_down_cb), 
 			  tab);
+	g_signal_connect (embed, "ge_contextmenu",
+			  GTK_SIGNAL_FUNC (galeon_tab_contextmenu_cb), 
+			  tab);
 	g_signal_connect (embed, "ge_security_change",
 			  GTK_SIGNAL_FUNC (galeon_tab_security_change_cb), 
 			  tab);
@@ -347,6 +353,8 @@ galeon_tab_finalize (GObject *object)
 	g_free (tab->priv->link_message);
 	g_free (tab->priv->js_message);
 
+        g_free (tab->priv->title);
+        g_free (tab->priv->location);
         g_free (tab->priv);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -853,13 +861,15 @@ galeon_tab_visibility_cb (GaleonEmbed *embed, gboolean visibility,
 #endif
 }
 
-static void
-galeon_tab_destroy_brsr_cb (GaleonEmbed *embed, GaleonTab *tab)
+void
+galeon_tab_destroy (GaleonTab *tab)
 {
 	GaleonWindow *window;
 
-	window = galeon_tab_get_window (tab);
+	if (tab->priv->closing) return;
+	tab->priv->closing = TRUE;
 
+	window = galeon_tab_get_window (tab);
 	galeon_window_remove_tab (window, tab);
 }
 
@@ -958,7 +968,7 @@ galeon_tab_dom_mouse_down_cb  (GaleonEmbed *embed,
 					       g_object_ref (event), 
 					       g_object_unref);
 			
-			galeon_embed_event_get_mouse_coords (event, &x, &y);
+			galeon_embed_event_get_coords (event, &x, &y);
 			gul_gestures_start (g, GTK_WIDGET (window), button, x, y);
 			
 			g_signal_connect (g, "gesture-performed", 
@@ -993,7 +1003,7 @@ galeon_tab_dom_mouse_down_cb  (GaleonEmbed *embed,
 		{
 			GulGestures *g = gul_gestures_new ();
 			guint x, y;
-			galeon_embed_event_get_mouse_coords (event, &x, &y);
+			galeon_embed_event_get_coords (event, &x, &y);
 			gul_gestures_start (g, GTK_WIDGET (window), button, x, y);
 			
 			g_signal_connect (g, "gesture-performed", 
@@ -1006,7 +1016,7 @@ galeon_tab_dom_mouse_down_cb  (GaleonEmbed *embed,
 		{
 			GaleonEmbedAutoscroller *as = galeon_embed_autoscroller_new ();
 			guint x, y;
-			galeon_embed_event_get_mouse_coords (event, &x, &y);
+			galeon_embed_event_get_coords (event, &x, &y);
 			galeon_embed_autoscroller_set_embed (as, embed);
 			galeon_embed_autoscroller_start_scroll (as, GTK_WIDGET (window), x, y);
 			g_object_unref (as);
@@ -1016,7 +1026,7 @@ galeon_tab_dom_mouse_down_cb  (GaleonEmbed *embed,
 		{
 			GaleonEmbedManualScroller *ms = galeon_embed_manual_scroller_new ();
 			guint x, y;
-			galeon_embed_event_get_mouse_coords (event, &x, &y);
+			galeon_embed_event_get_coords (event, &x, &y);
 			galeon_embed_manual_scroller_set_embed (ms, embed);
 			galeon_embed_manual_scroller_start_scroll (ms, GTK_WIDGET (window), x, y);
 			g_object_unref (ms);
@@ -1106,6 +1116,14 @@ galeon_tab_dom_mouse_click_cb(GaleonEmbed *embed,
 	return FALSE;
 }
 
+static void
+galeon_tab_contextmenu_cb (GaleonEmbed *embed,
+		           GaleonEmbedEvent *event,
+			   GaleonTab *tab)
+{
+	galeon_tab_show_embed_popup (tab, event);
+}
+
 static void
 galeon_tab_security_change_cb (GaleonEmbed *embed, EmbedSecurityLevel level,
 			       GaleonTab *tab)
@@ -1436,7 +1454,7 @@ galeon_tab_gesture_performed_cb (GulGestures *g, const gchar *sequence, GaleonTa
 	}
 	else if (!strcmp (action, "close"))
 	{
-		galeon_window_remove_tab (window, tab);
+		galeon_tab_destroy (tab);
 	}
 	else if (!strcmp (action, "back"))
 	{
