@@ -15,6 +15,9 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "history-dialog.h"
 #include "galeon-shell.h"
@@ -22,6 +25,7 @@
 #include "gul-string.h"
 #include "gul-gui.h"
 #include "galeon-dnd.h"
+#include "location-entry.h"
 
 #include <gtk/gtktreeview.h>
 #include <gtk/gtktreestore.h>
@@ -798,5 +802,6 @@ history_clear_button_clicked_cb (GtkWidget *button,
 {
 	history_dialog_clear_view (dialog);
 	global_history_clear (dialog->priv->gh);
+	galeon_location_entry_clear_history ();
 }
 
