@@ -329,11 +329,6 @@ window_cmd_bookmarks_edit (BonoboUIComponent *uic,
 	wbap = galeon_window_bookmark_activated_proxy_new ();
 	galeon_window_bookmark_activated_proxy_set_window (wbap, window);
 	galeon_window_bookmark_activated_proxy_connect (wbap, G_OBJECT (e));
-
-	g_signal_connect_swapped (gb_editor_get_main_widget (e), 
-				  "delete_event", G_CALLBACK (g_object_unref), e);
-	g_signal_connect_swapped (gb_editor_get_main_widget (e), 
-				  "delete_event", G_CALLBACK (g_object_unref), wbap);
 }
 
 void
@@ -819,6 +814,19 @@ window_cmd_settings_toolbar_editor (BonoboUIComponent *uic,
 	gchar *current_str;
 	GtkButton *revert_button;
 
+	if (tbe != NULL)
+	{
+		gul_tb_editor_set_parent (tbe, GTK_WIDGET(window));
+		gul_tb_editor_show (tbe);
+                return;
+	}
+
+	tbe = gul_tb_editor_new ();
+	g_object_add_weak_pointer (G_OBJECT (tbe),
+				   (gpointer)&tbe);
+	gul_tb_editor_set_parent (tbe, 
+				  GTK_WIDGET(window));
+
 	avail = gul_toolbar_new ();
 	gul_toolbar_parse (avail, AVAILABLE_TOOLBAR_ITEMS);
 
@@ -829,20 +837,6 @@ window_cmd_settings_toolbar_editor (BonoboUIComponent *uic,
 		gul_toolbar_parse (current, current_str);
 		g_free (current_str);
 	}
-
-	if (!tbe)
-	{
-		tbe = gul_tb_editor_new ();
-		g_object_add_weak_pointer (G_OBJECT (tbe),
-                                           (void **)&tbe);
-		gul_tb_editor_set_parent (tbe, 
-					  GTK_WIDGET(window));
-	}
-	else
-	{
-		gul_tb_editor_show (tbe);
-                return;
-	}
 	
 	gul_tb_editor_set_toolbar (tbe, current);
 	gul_tb_editor_set_available (tbe, avail);
