@@ -49,6 +49,7 @@ struct _GaleonLocationEntryPrivate {
 	gint show_alternatives_timeout;
 	gboolean block_set_autocompletion_key;
 	gboolean hack_for_going_to_the_site_when_clicking_in_the_location_history;
+	gboolean editing;
 
 	gchar *autocompletion_key;
 	gchar *last_completion;
@@ -57,6 +58,8 @@ struct _GaleonLocationEntryPrivate {
 #define AUTOCOMPLETION_DELAY 200
 #define SHOW_ALTERNATIVES_DELAY 500
 
+#define GALEON_LOCATION_ENTRY_HISTORY_ID	"galeon-url-history"
+
 /**
  * Private functions, only availble from this file
  */
@@ -74,6 +77,7 @@ static gint		galeon_location_entry_autocompletion_show_alternatives_to (GaleonLo
 static void		galeon_location_entry_autocompletion_window_url_activated_cb
 /***/								(GaleonAutocompletionWindow *aw, 
 								 const gchar *url, 
+								 gboolean new_tab_or_window,
 								 GaleonLocationEntry *w);
 static void		galeon_location_entry_list_event_after_cb (GtkWidget *list,
 								   GdkEvent *event, 
@@ -119,8 +123,16 @@ galeon_location_entry_class_init (GaleonLocationEntryClass *klass)
 		G_SIGNAL_RUN_FIRST | G_SIGNAL_RUN_LAST | G_SIGNAL_RUN_CLEANUP,
                 G_STRUCT_OFFSET (GaleonLocationEntryClass, galeon_location_entry_url_activated), 
 		NULL, NULL, 
-		galeon_marshal_VOID__STRING,
-		G_TYPE_NONE, 1, G_TYPE_STRING);
+		galeon_marshal_VOID__STRING_INT,
+		G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_BOOLEAN);
+}
+
+static gboolean
+focus_out_event_cb (GtkWidget *entry, GdkEventFocus *event, 
+		    GaleonLocationEntry *w)
+{
+	w->priv->editing = FALSE;
+	return FALSE;
 }
 
 static void 
@@ -128,8 +140,11 @@ galeon_location_entry_init (GaleonLocationEntry *w)
 {
 	GaleonLocationEntryPrivate *p = g_new0 (GaleonLocationEntryPrivate, 1);
 	w->priv = p;
-		
+
 	galeon_location_entry_build (w);
+
+	g_signal_connect (w->priv->entry, "focus_out_event",
+			  G_CALLBACK (focus_out_event_cb), w);
 }
 
 static void
@@ -162,13 +177,23 @@ galeon_location_entry_new (void)
 	return GALEON_LOCATION_ENTRY (g_object_new (GALEON_TYPE_LOCATION_ENTRY, NULL));
 }
 
+void
+galeon_location_entry_clear_history (void)
+{
+	GtkWidget *w;
+
+	w = gnome_entry_new (GALEON_LOCATION_ENTRY_HISTORY_ID);
+	gnome_entry_clear_history (GNOME_ENTRY (w));
+	gtk_widget_destroy (w);
+}
+
 static void
 galeon_location_entry_build (GaleonLocationEntry *w)
 {
 	GaleonLocationEntryPrivate *p = w->priv;
 	GtkWidget *list;
 
-	p->combo = gnome_entry_new ("galeon-url-history");
+	p->combo = gnome_entry_new (GALEON_LOCATION_ENTRY_HISTORY_ID);
 	p->entry = GTK_COMBO (p->combo)->entry;
 	gtk_widget_show (p->combo);
 	gtk_box_pack_start (GTK_BOX (w), p->combo, TRUE, TRUE, 0);
@@ -299,14 +324,34 @@ galeon_location_entry_autocompletion_to (GaleonLocationEntry *w)
 	return FALSE;
 }
 
+static void
+real_set_location (GaleonLocationEntry *w,
+		   const gchar *new_location)
+{
+	GaleonLocationEntryPrivate *p = w->priv;
+	int pos = 0;
+	gtk_editable_delete_text (GTK_EDITABLE (p->entry), 0, -1);
+	gtk_editable_insert_text (GTK_EDITABLE (p->entry), new_location, 
+			          strlen (new_location), &pos);
+}
+
+static void
+emit_activated_signal (GaleonLocationEntry *w, const char *url, gboolean new_tab_or_window)
+{
+	w->priv->editing = FALSE;
+
+	g_signal_emit (w, GaleonLocationEntrySignals[GALEON_LOCATION_ENTRY_URL_ACTIVATED], 
+                       0, url, new_tab_or_window);
+}
+
+
 /* this is from the old location entry, need to do the autocompletion before implementing this */
 static gboolean
 galeon_location_entry_key_press_event_cb (GtkWidget *entry, GdkEventKey *event, GaleonLocationEntry *w)
 {
 	GaleonLocationEntryPrivate *p = w->priv;
         static gboolean suggest = FALSE;
-        guint keyval = event->keyval;
-	
+
         if (p->autocompletion_timeout != 0)
 	{
                 gtk_timeout_remove (p->autocompletion_timeout);
@@ -325,11 +370,26 @@ galeon_location_entry_key_press_event_cb (GtkWidget *entry, GdkEventKey *event,
                 suggest = FALSE;
 	}
 	
-        if (((event->state & GDK_Control_L || event->state & GDK_Control_R) &&
-             (keyval == GDK_a || keyval == GDK_b || keyval == GDK_c ||
-              keyval == GDK_d || keyval == GDK_e || keyval == GDK_f ||
-              keyval == GDK_h || keyval == GDK_k || keyval == GDK_u ||
-              keyval == GDK_v || keyval == GDK_w || keyval == GDK_x)) ||
+
+	/* Handle any CTRL+Enter events */
+	if ( event->state & GDK_CONTROL_MASK && 
+	     ( event->keyval == GDK_KP_Enter || event->keyval == GDK_Return ) )
+	{
+		char *url = gtk_editable_get_chars (GTK_EDITABLE(entry), 0, -1);
+
+		DEBUG_MSG (("In galeon_location_key_press_cb, going to %s\n", url));
+		   
+		galeon_location_entry_autocompletion_hide_alternatives (w);
+
+		emit_activated_signal (w, url, TRUE);
+
+		g_free (url);
+
+		return TRUE;
+	}
+
+	/* Ingore any CTRL+<key> commands (or backspace commands) */
+        if ((event->state & GDK_CONTROL_MASK) ||
             (event->state == 0 && event->keyval == GDK_BackSpace))
         {
 		galeon_location_entry_autocompletion_hide_alternatives (w);
@@ -403,12 +463,22 @@ galeon_location_entry_key_press_event_cb (GtkWidget *entry, GdkEventKey *event,
 
                         p->block_set_autocompletion_key = FALSE;
 
-			galeon_location_entry_autocompletion_show_alternatives (w);
+			/* show alternatives list only if there is no
+			 * non-ambiguous prefix we can complete
+			 */
                         if (!strcmp (common_prefix, text))
                         {
+				galeon_location_entry_autocompletion_show_alternatives (w);
                                 /* really suggest something the next time */
                                 suggest = TRUE; 
                         }
+			else if (p->autocompletion_window_visible)
+			{
+				/* if the alternatives list is already visible,
+				 * update its contents to match the new prefix
+				 */
+				galeon_location_entry_autocompletion_show_alternatives (w);
+			}
                         g_free (common_prefix);
                 }
                 else
@@ -422,7 +492,7 @@ galeon_location_entry_key_press_event_cb (GtkWidget *entry, GdkEventKey *event,
 		galeon_location_entry_autocompletion_hide_alternatives (w);
                 if (p->before_completion)
                 {
-                        galeon_location_entry_set_location (w, p->before_completion);
+                        real_set_location (w, p->before_completion);
                         g_free (p->before_completion);
                         p->before_completion = NULL;
 			gtk_editable_set_position (GTK_EDITABLE (p->entry), -1);
@@ -430,10 +500,14 @@ galeon_location_entry_key_press_event_cb (GtkWidget *entry, GdkEventKey *event,
                 }
                 else
                 {
-                        //galeon_location_entry_set_location (w, embed->location);
+                        //real_set_location (w, embed->location);
                 }
                 break;
         default:
+		/* ignore keys that don't generate content (Shift, Ctrl, ...) */
+		if (event->length <= 0) break;
+
+		w->priv->editing = TRUE;
 		galeon_location_entry_autocompletion_unselect_alternatives (w);
 		if ((eel_gconf_get_boolean (CONF_COMPLETION_SHOW_LIST_AUTO))
 		    && (event->string[0] > 32) && (event->string[0] < 126))
@@ -467,7 +541,8 @@ galeon_location_entry_activate_cb (GtkEntry *entry, GaleonLocationEntry *w)
 
 	DEBUG_MSG (("In galeon_location_entry_activate_cb, going to %s\n", url));
 
-	g_signal_emit (w, GaleonLocationEntrySignals[GALEON_LOCATION_ENTRY_URL_ACTIVATED], 0, url);
+	emit_activated_signal (w, url, FALSE);
+
 	g_free (url);
 }
 
@@ -501,11 +576,10 @@ void
 galeon_location_entry_set_location (GaleonLocationEntry *w, 
 				    const gchar *new_location)
 {
-	GaleonLocationEntryPrivate *p = w->priv;
-	int pos;
-	gtk_editable_delete_text (GTK_EDITABLE (p->entry), 0, -1);
-	gtk_editable_insert_text (GTK_EDITABLE (p->entry), new_location, g_utf8_strlen (new_location, -1),
-				  &pos);
+	if (!w->priv->editing)
+	{
+		real_set_location (w, new_location);
+	}
 }
 
 gchar *
@@ -556,15 +630,17 @@ galeon_location_entry_set_autocompletion (GaleonLocationEntry *w,
 
 static void
 galeon_location_entry_autocompletion_window_url_activated_cb (GaleonAutocompletionWindow *aw, 
-							      const gchar *url, GaleonLocationEntry *w)
+							      const gchar *url, 
+							      gboolean new_tab_or_window,
+							      GaleonLocationEntry *w)
 {
-	galeon_location_entry_set_location (w, url);
+	real_set_location (w, url);
 
 	DEBUG_MSG (("In location_entry_autocompletion_window_url_activated_cb, going to %s\n", url));
 
 	galeon_location_entry_autocompletion_hide_alternatives (w);
 
-	g_signal_emit (w, GaleonLocationEntrySignals[GALEON_LOCATION_ENTRY_URL_ACTIVATED], 0, url);
+	emit_activated_signal (w, url, new_tab_or_window);
 }
 
 static void
@@ -591,19 +667,13 @@ galeon_location_entry_autocompletion_window_hidden_cb (GaleonAutocompletionWindo
 }
 
 void
-galeon_location_entry_activate (GaleonLocationEntry *w)
+galeon_location_entry_edit (GaleonLocationEntry *w)
 {
-	GtkWidget *toplevel;
-
-	toplevel = gtk_widget_get_toplevel (w->priv->entry);
-	
-	gtk_editable_select_region (GTK_EDITABLE(w->priv->entry),
-                 		    0, -1);
-        gtk_window_set_focus (GTK_WINDOW(toplevel),
-                              w->priv->entry);
+	w->priv->editing = TRUE;
+	/* gtk_editable_select_region (GTK_EDITABLE(w->priv->entry), 0, -1); */
+	gtk_widget_grab_focus (w->priv->entry);
 }
 
-
 static void
 galeon_location_entry_list_event_after_cb (GtkWidget *list,
 					   GdkEvent *event, 
@@ -629,8 +699,7 @@ galeon_location_entry_editable_changed_cb (GtkEditable *editable, GaleonLocation
 		if (url && url[0] != '\0')
 		{
 			p->hack_for_going_to_the_site_when_clicking_in_the_location_history = FALSE;
-			g_signal_emit 
-				(e, GaleonLocationEntrySignals[GALEON_LOCATION_ENTRY_URL_ACTIVATED], 0, url);
+			emit_activated_signal (e, url, FALSE);
 			g_free (url);
 		}
 	}
